/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS, 
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details. 
 * The XtreemOS project has been developed with the financial support of the European 
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License as published by the Free Software Foundation, 
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS. 
 * If not, see <http://www.gnu.org/licenses/>.
 * 
 * */


/** @file map_mech.h
 * @brief The implementation of user mapping.
 *
 * The file implements the mapping mechanism from global user to local user. 
 * Mapping user includes 
 *  a) mapping the global identity token (e.g. DN,VO,ROLE) to local identity 
 *     token which contains one local uid and one or more local gid.
 *  b) maintaining the one-by-one relationship between global and local.
 * 
 */
#ifndef _MAP_MECH_H_
#define _MAP_MECH_H_

#include "xos_dbaux.h"
#include "xos_policy_rule.h"

#define MAXBUF		128    /**< maximal size of buffer */
#define MAXLINE		256    /**< maximal length of string */
#define RANDSTR_LEN	4      /**< default length of random string */

/* mapping mode
 * To defne how to do mapping.
 * To-do: redefine the mapping rules 
 */
#define M_EXACT		0      /**< Mapping mode: exact mapping */
#define M_BULK		1      /**< Mapping mode: bulk mapping */
#define M_QUOTA		2      /**< Mapping mode: limited mapping */

/* mapping type
 * To define the type of mapping target.
 */
#define T_USER		5      /**< Mapping type: exact account or group */
#define	T_POOL		6      /**< Mapping type: account pool */
#define T_ARBITRA	7      /**< Mapping type: arbitrary */

/* error codes */
#define E_NOEXIST	1      /**< Error code: Not found or Not exist */
#define E_NOACCUNT	2      /**< Error code: Error in geting account */
#define E_NEXTRULE	3      /**< Error code: Invalid rule */
#define E_PASSWD_SYS	4      /**< Error code: Errors in accessing /etc/passwd */
#define E_GROUPS_SYS	5      /**< Error code: Errors in accessing /etc/group */
#define E_ERRORS	6      /**< Error code: Other errors */
/** Default location of all the configure files */
#define DEFAULT_MAPFILEDIR	"/etc/xos/maprule/"
/** Configure file of account mapping */
#define AMAPPEDFILE 		"amappedfile"
/** Configure file of group mapping */
#define GMAPPEDFILE 		"gmappedfile"
/** Configure file of basical VO setting */
#define POLICYFILE 		"policy.conf"

/** @brief structure of mapping rule.
 * 
 * The structure holds the mapping rule parsed from configure files. 
 */
typedef struct _mapped_token {
	char mapped_from[MAXFIELD];/**< Input of mapping venter */
	char mapped_to[MAXFIELD];  /**< Output of mapping venter */
	int mapped_type;	   /**< Mapping type belonged to */
	int mapped_mode;	   /**< Mapping mode belonged to */

} MAPTOKEN;


/** @brief structure of policy
 *
 * Currently, configure items defined in policy are mapping boundary
 * of uid and gid.
 */
typedef struct _policy_token {
	int uid_max;/**< Maxmium of uid */
	int uid_min;/**< Minmium of uid */
	int gid_max;/**< Maxmium of gid */
	int gid_min;/**< Minmium of gid */

} PLYTOKEN;

/** allocate a new MAPTOKEN struct
 */
MAPTOKEN *mtoken_new();

/** allocate a new PLYTOKEN struct
 */
PLYTOKEN *ptoken_new();



/** Mapping user's global identity to local group
 */
int groupmapping(char *, char *, GGROUPS *);

/** Mapping user's global identity to local account
 */
int accountmapping(char *, char *, char *, GPASSWD *);

/** Mapping user's global identity to local one
 */
int xos_mapping(char *, char *, char *, char *, GPASSWD *, GGROUPS *);

/** Clear mapped account in database
 */
int clear_account(GPASSWD *);

/** Clear mapped group in database
 */
int clear_group(GGROUPS *);

/** Clear all the item in database
 */
int clear_all();


int find_matchrule_mapping(int rtype, char *sub_1, char *sub_2,
			   MAPTOKEN * mtk);


#endif				/* MAP_MECH_H */
