/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */


/** @file xos_ams.h
 * @brief client interfaces
 *
 * The file aims at implementing the AMS client codes, which provide connecting and
 * transmitting interfaces with AMS server.
 */

#ifndef _XOS_AMS_H_
#define _XOS_AMS_H_

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <time.h>

#include "xos_dbaux.h"
#include "xos_policy_rule.h"


#define AMS_GPASSWD	GPASSWD
#define AMS_GGROUPS	GGROUPS


/** Client interface for mapping: Mapping user
 *  The function will pass the identity information to AMS server and told it
 * to finish mapping task.
 * @param mapped_dn the mapped DN information
 * @param mapped_vo the mapped VO information
 * @param mapped_role the mapped ROLE information
 * @param mapped_subgroups the mapped SUBGROUPS information
 * @param gpwd the returned GPASSWD holding the mapped user information
 * @param primarygrp the returned GGROUPS holding the mapped group information
 * @return zero if success, or non-zero.
 */
int amsclient_usermapping(char *mapped_dn, char *mapped_vo,
			  char *mapped_role, char *mapped_subgroup,
			  AMS_GPASSWD * gpwd, AMS_GGROUPS * primarygrp);

/** Client interface for mapping: Get mapping information according grid 
 * identity information.
 * @param mapped_dn the mapped DN information
 * @param mapped_vo the mapped VO information
 * @param mapped_subgroup the mapped ROLE/SUBGROUPS information
 * @param gpwd the returned GPASSWD holding the mapped user information
 * @param primarygrp the returned GGROUPS holding the mapped group information
 * @return zero if success, or non-zero
 * 
 */
int amsclient_mappinginfo(char *mapped_dn, char *mapped_vo,
			  char *mapped_subgroup, AMS_GPASSWD * gpwd,
			  AMS_GGROUPS * primarygrp);

/** Client interface for mapping: Clear mapping information according to 
 * grid identity.
 * @param mapped_dn the mapped DN information
 * @param mapped_vo the mapped VO information
 * @param mapped_role the mapped ROLE/SUBGROUP information
 * @return zero if success, or non-zero
 */
int amsclient_clearmapping(char *mapped_dn, char *mapped_vo,
			   char *mapped_subgroup);

/** Client interface for mapping: Clear mapping information according to 
 * local uid / gid .
 * @param clear_uid the uid of local mapped account
 * @param clear_gid the gid of local mapped account
 * @return zero if success, or non-zero
 */
int amsclient_clearuidgid(int clear_uid, int clear_gid);

/** Client interface for mapping: Invert user mapping
 * The function will pass local user identity information and get back 
 * the grid identity nformation.
 * 
 * @param username the username of local mapped account
 * @param user_uid the uid of local mapped account
 * @param groupname the groupname of local mapped group
 * @param group_gid the gid of local mapped group
 * @param gpwd the returned GPASSWD holding the mapped user information
 * @param ggrp the returned GGROUPS holding the mapped group information 
 * @return zero if success, or non-zero
 */
int amsclient_invmappinginfo(char *username, int user_uid, char *groupname,
			     int group_gid, AMS_GPASSWD * gpwd,
			     AMS_GGROUPS * ggrp);

/* Expired ?
 * Policy handlers should be put in other service if the functionalities of policy 
 * management is sperated from AMS, to another server such as AES */

/** Client interface: Policy translation
 * The interface will call AMS to do policy translation and setup local quota limitation
 * according to local policy mapping rule
 * @param username the local mapped name
 * @param attrs the attributes information, string type
 * @return zero if success, or non-zero
 */
int amsclient_policytranslate(char *username, char *attrs);

/** Client interface: Policy clear
 * The interface will call AMS to clear quota limitation of specific user,
 * according to his attributes.
 * @param username the local mapped name
 * @param attrs the attributes information
 * @return zero if success, or non-zero
 */
int amsclient_policyclear(char *username, char *attrs);

/** Client interface: Clear policy in given server.
 * @param username the local mapped name
 * @param attrs the attributes information
 * @param ams_server the IPaddress of AMS
 * @param ams_port the port of AMS
 * @return zero if successful, or non-zero
 * */
int amsclient_policyclear2(char *username, char *attrs, char *ams_server,
			   int ams_port);

/** client interface for temp data: save temporary information to global database
 * taken charge by AMS. 
 * Notice: the temporary information can only be used onece. After 
 * fetching by amsclient_tmpinfoget(), the data will be removed from
 * global database.
 * 
 * @param tmpinfo the buffer of temporary information.
 * @param reky the key to retrive back information. The key buffer should
 * 	be allocated in caller.
 * @return zero if successful, or non-zero.
 * */
int amsclient_tmpinfosave(char *tmpinfo, char *rkey);

/** client interface for temp data: get temporary information from global database
 * taken charge by AMS.
 * Notice: the data will be cleared in the same time after the operation
 * finish.
 * @param key the key to retrieve data.
 * @param tmpinfo the pointer to hold returned data from global database.
 * 	If fail, NULL will be set as returned value.
 * @return zero if successful, or non-zero.
 * */
int amsclient_tmpinfoget(char *key, char **tmpinfo);


/* ams client for managing policy rules */

/** client interface for rule management: put rule into rule database.
 * @param xrule the rule will be stored.
 * @param handler the handler of stored rule. If faul, NULL will be presented
 * @return zero if successful, or non-zero.
 * */
int amsclient_ruleput(struct rule_t *xrule, char **handler);

/** client interface for rule management: get rule by key.
 * The return values includes rule handler besides rule data.
 * @param xrule the porinter to hold rule data.
 * @param handler the handler pointer to be stored the rule handler. If the 
 * 	parameter is NULL, nothing is given.
 * @return zeor if successful, or non-zero.
 * */
int amsclient_getrulebykey(unsigned int skey, struct rule_t **xrule,
			   char **handler);

/** client interface for policy management: get rule by handler.
 * @param skey  the handler of rule.
 * @param xrule the pointer to hold returned rule data.
 * @return zero if successful, or non-zoer.
 * */
int amsclient_getrulebyhandler(char *skey, struct rule_t **xrule);

/** client interface for rule management: delete rule by handler
 * @param handler the buffer holding the handler of will-be deleted rule.
 * @return zero if successful, or non-zero.
 * */
int amsclient_ruledelete(char *handler);

/** client interface for rule management: search given rule.
 * @param xrule the buffer holding the rule data
 * @param handler the pointer to return the handler of given rule.
 * @return zero if successful, or non-zero.
 * */
int amsclient_rulesearch(struct rule_t *xrule, char **handler);

/** client interface for rule management: check whether user is
 * in blacklist.
 * @praam sub  the subject holding given user identity.
 * @param in_blacklist the flag returned to show whether in blacklist.
 * @return zero if operation is succesful, or non-zero.
 * */
int amsclient_inblacklist(struct subject_t *sub, int *in_blacklist);

/** client interface for rule management: get the rules which can 
 * be applied to given user. The rules will be presented as the format
 * which is corresponding to given user.
 * @param username  local user name or mapped local user name.
 * @param rule_buf the buffer to hold the applied rule.
 * @return zero if successful, or non-zero. 
 * */
int amsclient_enforcedrules(char *username, char *rule_buf);

/** client interface for changing user's local home directory
 * @param luid the local uid of grid user
 * @param hm_dir home directory need to be changed to.
 * @return zero if successfule, or non-zero.
 * */
int amsclient_changeuserinfo(int luid, char *hm_dir);

/** client interface for operating the number of mount point of GPASSWD
 * @param opt_cmd internal operation commend. The internal commands includes:
 * 	"inc" - add 1 to current mount point
 * 	"dec" - decrease 1 to current mount point
 * 	"set" - set the parameter to current mount point
 * 	"get" - get current the mount number of GPASSWD 
 * @param user_uid the user's local id for operation
 * @param mount_para the parameter for operation
 * @return "get" - number of mount point; other - zero if sucess, or non-zero
 * */
int amsclient_optmountnum(char *opt_cmd, int user_uid, int mount_para);

/* internal functions 
 * 
 * Added the connection handler to support multiple operations
 * in one AMS connection.
 *
 * */

int amsclient_usermapping_internal(int sockfd, char *mapped_dn,
				   char *mapped_vo, char *mapped_role,
				   char *mapped_subgroup,
				   AMS_GPASSWD * gpwd,
				   AMS_GGROUPS * primarygrp);


int amsclient_mappinginfo_internal(int sockfd, char *mapped_dn,
				   char *mapped_vo, char *mapped_subgroup,
				   AMS_GPASSWD * gpwd,
				   AMS_GGROUPS * primarygrp);

int amsclient_clearmapping_internal(int sockfd, char *mapped_dn,
				    char *mapped_vo,
				    char *mapped_subgroup);

int amsclient_clearuidgid_internal(int sockfd, int clear_uid,
				   int clear_gid);

int amsclient_invmappinginfo_internal(int sockfd, char *username,
				      int user_uid, char *groupname,
				      int group_gid, AMS_GPASSWD * gpwd,
				      AMS_GGROUPS * ggrp);

int amsclient_policytranslate_internal(int sockfd, char *username,
				       char *attrs);

int amsclient_policyclear_internal(int sockfd, char *username,
				   char *attrs);

int amsclient_tmpinfosave_internal(int sockfd, char *tmpinfo, char *rkey);

int amsclient_tmpinfoget_internal(int sockfd, char *key, char **tmpinfo);

int amsclient_ruleput_internal(int sockfd, struct rule_t *xrule,
			       char **handler);

int amsclient_getrulebykey_internal(int sockfd, unsigned int skey,
				    struct rule_t **xrule, char **handler);

int amsclient_getrulebyhandler_internal(int sockfd, char *skey,
					struct rule_t **xrule);

int amsclient_ruledelete_internal(int socket, char *handler);

int amsclient_rulesearch_internal(int sockfd, struct rule_t *xrule,
				  char **handler);

int amsclient_inblacklist_internal(int sockfd, struct subject_t *sub,
				   int *in_blacklist);

int amsclient_enforcedrules_internal(int sockfd, char *username,
				     char *rule_buf);

int amsclient_changeuserinfo_internal(int sockfd, int luid, char *hm_dir);

int amsclient_optmountnum_internal(int sockfd, char *opt_cmd, int user_uid, int mount_para);
#endif				/* _XOS_AMS_H_ */
