/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/** @file xos_policy_db.h
 * @brief interface file for policy-related databases
 *  
 * The policy-related databases is stored in four categories
 * of rules, involving mapping, resource usage, access control,
 * and application-specific rules.
 *
 * Similarly as GPASSWD and GGROUPS, the databases is maintained
 * by AMS.
 * 
 * */
#ifndef _XOS_POLICY_DB_H_
#define _XOS_POLICY_DB_H_

#include "xos.h"
#include "bdb_common.h"
#include "xos_dbaux.h"
#include "xos_policy_rule.h"

/** Default directory for rule database*/
#define	RULEDB_DIR	DB_DEFAULT_TMP_DIR
/** name of rule database */
#define	RULEINFO_DB	"ruleinfo.db"
/** index database */
#define	RULEINDEX_DB	"ruleindex.db"


/** data structure of rule databases */
typedef struct _ruledbs {
	DB *ruleinfo_dbp;  /**< pointing to database file*/
	DB *ruleindex_dbp; /**< pointing to index file */

	char *ruledb_home_dir; /**< home directory of database files */
	char *ruleinfo_db_name; /**< database name */
	char *ruleindex_db_name; /**< index database name */
} RULEINFO_DBS;

/** intiate rule databases  
 * @param ruledbp The pointer of above data structure
 * @return zero if successul, or non-zero
 * */
int ruleinfo_db_init(RULEINFO_DBS * ruledbp);

/** close rule databases 
 * @param ruledbp The pointer of data structure
 * @return zero if successful, or non-zeor
 * */
int ruleinfo_db_release(RULEINFO_DBS * ruledbp);

/** get rule from database by integer-type key
 * @param skey the key value
 * @param xrule the pointer of rule structure. If successful, xrule will
 * 	hold the rule data, or NULL will be set as value. In the same 
 * 	time, handler will be the handler of returned rule data.If the 
 * 	field is NULL, nothing will be returned.
 * @param handler the handler of xrule.If it is NULL, nothing will be
 * 	returned.
 * @return zero if sucessful, or non-zero
 * */
int ruleinfo_db_getbykey(unsigned int skey, struct rule_t **xrule,
			 char **handler);

/** get rule data by handler
 * @param skey the hanler painter of rule
 * @param xrule the returned rule data. If fail, xrule will be set with
 * 	NULL
 * @return zero if successful, or non-zero
 * */
int ruleinfo_db_getbyhdler(char *skey, struct rule_t **xrule);

/** put rule into database
 * @param xrule the pointer of rule data, which should be valid
 * 	pointer of struct rule_t.
 * @param handler to return the handler of stored rule. 
 * @return zero if successful, or non-zeor.
 **/
int ruleinfo_db_put(struct rule_t *xrule, char **handler);

/** put the rule to given location or replace the rule with given key.
 * @param xrule The pointer of rule structure.
 * @param given_key the key (location or priviority) of database.
 * @param handler the returned handler of stored rule.
 * @return zero if sucessful, or non-zero.
 * */
int ruleinfo_db_put_aux(struct rule_t *xrule, int given_key,
			char **handler);

/** delete the rule with given handler
 * @param handler the handler of will-be deleted rule.
 * @return zero if successful, or non-zero
 * */
int ruleinfo_db_delete(char *handler);

/** print all the records in database
 * */
void ruleinfo_db_printdb();

/** print specific rule in database
 * @param xrule the pointer of rule needed to be print.
 * */
void ruleinfo_db_printdata(struct rule_t *xrule);

/** clear all the items in database
 * @return zero if successful, or non-zero
 * */
int ruleinfo_db_clearall();

/** search given rule data and give back its handler if the
 * rule is existing in database. The search is according
 * to rule content.
 * @param xrule the pointer of rule needed to be searched. 
 * @param handler the retuened handler if the rule is in 
 * 	database.
 * @return zero if successful, or non-zero.
 * */
int ruleinfo_db_search(struct rule_t *xrule, char **handler);

/** return current maximal key
 * @return integer value of current maximal key + 1.
 * */
unsigned int ruleinfo_db_newhandler();

/** change the location of rule with given key.
 * Because the location is defining the priority of rule,
 * the interface can also be used to adjust the rule priority.
 * @parem pre_key the key (location or priority) of current
 * 	rule.
 * @parem cur_key new key (location or priority) expected to
 * 	change.
 * @return zero if successful, or non-zero.
 * */
int ruleinfo_db_change_location(int pre_key, int cur_key);

#endif				/* _XOS_POLICY_DB_H_ */



