/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/** @file xos_readconf.h
 * @brief Read and parse configure file of PAM module
 *
 * The file defines the interfaces to read and parse
 * configure parameters defined in disk file. The 
 * configure parameters are used in setup and adjust
 * the working mode of PAM module.
 *
 * PAM module is designed with serveal working modes.
 * Different working mode need different configure 
 * parameters which defined in PAM_CONF in xos.h.
 *
 * The result of disposing configure file is a structure
 * containing enough information (as parameters). PAM 
 * module will work according to the parameters setting
 * in the structure.
 */
#ifndef _XOS_READCONF_H_
#define _XOS_READCONF_H_

#include "xos.h"

/* working parts */
#define USE_SHARE_BUFFER		0x10000	   /**< Running functionality of checking package secure transmision*/
#define	USE_CERTIFICATE_CHECK		0x01000	   /**< Running functionality of ckecking certificate */
#define	USE_EXTERNAL_AMS		0x00100	   /**< Running functionality of using external mapping server */
#define	USE_EXTERNAL_AES		0x00010	   /**< Running functionality of using external enforcement server */
#define USE_DEBUG			0x00001	   /**< debug mode, out of date */

/* working mode */
#define M_REMOTEUNSECURELOGIN		0x100000   /**< Working mode: connect to unsecure remote login shell */
#define	M_USESHAREBUFFER		0x010000   /**< Working mode: connect to secure remote login shell */
#define	M_MYPROXY			0x001000   /**< Working mode: MyProxy mode */
#define	M_LOGINSECURELOGIN		0x000100   /**< Working mode: local secure login */
#define	M_USEEXTERNALAMS		0x000010   /**< Working mode: external mapping server */
#define M_USEEXTERNALAES		0x000001   /**< Working mode: external enforcement server */


#define USE_PASSPHRASE			1	   /**< check user identity by prompting password */
#define	USE_LOCALCERT			2	   /**< check user identity by user's local certificate */

/** @brief data structure of configure parameters.
 * The structure is returned after parsing the configure disk.
 */
typedef struct {
	int mode;
	   /**< working mode */
	int use_debug;	/**< open debug information or not */
	int use_syslog;	/**< use the syslog to record*/
	int use_automount; /**< use the automount mechanism */
	int use_fixedidalloc; /**< use the fixed allocated uid/gid*/
	int use_cgroups;	/*< use kernel cgroup supporting process management*/

	char *myproxy_server;
		       /**< address of MyProxy server */
	int myproxy_port;
		   /**< port of MyProxy server */
	int usepass;
	      /**< whether using password or certificate */
        int ams_server_is_local;
              /**< the ams server is local and use a unix socket instead of tcp socket */
	char *ams_server;
		   /**<address of external mapping server */
	int ams_port;
	       /**< port of external mapping server */
	char *aes_server;
		   /**< address of external enforcement server */
	int aes_port;
	       /**< port of external enforcement server */
	char *vo_acconf;
		  /**< VO policy configure file*/
	char *vo_cacertdir;
		     /**< location of CA certificate */
	char *vo_cacertf;
			/**< certificate name of CDA*/
	char *node_prvkeyf;
		     /**< location of node's certificate */
	char *cgroup_subsys; /**< cgroup subsystems will be mounted */
} ConfData;

/** Default configure file of VO Policy*/
#define VO_ACCONF		DB_DEFAULT_TMP_DIR
/** Default CA location */
#define	VO_CAPUBKEYF		"/etc/security/certificates/"
/** Default CA certificate name */
#define VO_CACERTF		"xtreemos-ca.crt"
/** Default location of node's certificate*/
#define NODE_PRVKEYF		"/tmp/node-security/"
/** Defautl subsystem to be mounted in cgroup support */
#define CGROUP_SUBSYS_DEFAULT	"memory"

/**Initialize the ConfData struct*/
void initialize_confdata(ConfData *);
/** Fill the default value in ConfData struct*/
void fill_default_confdata(ConfData *);
/** Print confData struct parsed from configure disk*/
void print_confdata(ConfData *, int);
/**Reads the config file and modifies the confdata accordingly.  
  ConfData should already be initialized before this call*/
int xos_read_config_file(const char *, ConfData *, int);

#endif				/* _XOS_READCONF_H */
